/*

HXED, Version 2.00, a Hexadecimal Editor for Binary Files
Copyright 1991 Anthony F. Stuart, All rights reserved.

Date    Name	Change
910209	AFS	Initial release

*/


#include <sys/types.h>

#include "sd.h"
#include "def.h"
#include "ext.h"
#include "mcb.h"
#include "fmt.h"
#include "usage.h"
#include "proto.h"

x_addr	 old_addr;
x_addr	 old_size;
int	 old_insmode;
int	 old_hexmode;

/* Initialize header values and display header template */

display_header ()
{
	/* The initial settings reflect the numbers in the format string */

	old_addr = 0;
	old_size = 0;
	old_insmode = 0;
	old_hexmode = 0;

	/* Display the header with values initialized to those above */

	display_string (header_row, header_col, hdr_fmt, header_attr);
}

#include <time.h>

/*  Initialize and display the footer */

display_footer ()
{
	char		 time_buf[9];
	char		 date_buf[9];
	char		 size_buf[9];
	struct tm	*tm;

	strcpy (time_buf, "00.00.00");
	strcpy (date_buf, "00/00/00");

	/* Format date and time last modified and file size */

	tm = localtime (&(mcb->base_mtime));
	format_date_time (time_buf, tm->tm_hour, tm->tm_min, tm->tm_sec);
	format_date_time (date_buf, tm->tm_mon+1, tm->tm_mday, tm->tm_year);
	format_addr (size_buf, mcb->original_len);
	size_buf[nibbles_per_addr] = EOS;

	/* Display file statistics */

	display_string (footer_row, footer_col, ftr_fmt, footer_attr);
	display_string (footer_row, ftr_name_col, mcb->base_file, footer_attr);
	display_string (footer_row, ftr_time_col, time_buf, footer_attr);
	display_string (footer_row, ftr_date_col, date_buf, footer_attr);
	display_string (footer_row, ftr_size_col, size_buf, footer_attr);
}

/* Format date or time values and store in template */

format_date_time (template, a, b, c)
char	*template;
int	 a;
int	 b;
int	 c;
{
	template[0] = '0' + a / 10;
	template[1] = '0' + a % 10;
	template[3] = '0' + b / 10;
	template[4] = '0' + b % 10;
	template[6] = '0' + c / 10;
	template[7] = '0' + c % 10;
	template[8] = 0;
}

/* Update any fields in the header that have changed */

update_header ()
{
	char	*mode_name;
	char	 buf[BUF_SZ];

	/* Update current location if changed */

	if (this_addr != old_addr) {
		old_addr = this_addr;
		format_addr (buf, this_addr);
		buf[nibbles_per_addr] = EOS;
		display_string (header_row, hdr_addr_col, buf, header_attr);
	}

	/* Update current file size if changed */

	if (mcb->len != old_size) {
		old_size = mcb->len;
		format_addr (buf, mcb->len);
		buf[nibbles_per_addr] = EOS;
		display_string (header_row, hdr_size_col, buf, header_attr);
	}

	/* Update insert/overtype mode flag if changed */

	if (insert_mode != old_insmode) {
		old_insmode = insert_mode;
		mode_name = insert_mode ? "  Insert" : "Overtype";
		display_string (header_row, hdr_insmode_col, mode_name, header_attr);
	}

	/* Update hex/ascii mode flag if changed */

	if (hex_mode != old_hexmode) {
		old_hexmode = hex_mode;
		mode_name = hex_mode ? "  Hex" : "Ascii";
		display_string (header_row, hdr_hexmode_col, mode_name, header_attr);
	}
}

/* Display a page in hex/ascii dump format */

display_page (new_addr)
x_addr	 new_addr;
{
	int	 x;
	int	 bc;
	char	 out_buf [OUT_BUF_SZ];

	base_addr = new_addr;

	for (x = 0; x < lines_per_page; x++) {
		format_line (out_buf, new_addr);
		display_string (text_row+x, text_col, out_buf, text_attr);
		new_addr += bytes_per_line;
	}
}

/* Format a line in hex/ascii dump format */

format_line (out_buf, line_addr)
char	*out_buf;
x_addr	 line_addr;
{
	int	 y;
	int	 c;
	char	*sp;
	char	*tp;

	/* Initialize output string with a copy of the template string */

	tp = out_buf;
	sp = fmt_buf;

	while (*sp) {
		*tp++ = *sp++;
	}

	*tp++ = EOS;

	/* Format line address and store in output string */

	format_addr (out_buf, line_addr);

	/* Format the hex and ascii data for each byte in the row */

	for (y = 0; y < bytes_per_line; y++) {
		c = get_byte (line_addr++);
		if (c == EOF)
			return (y);
		out_buf[hex_map[y]] = get_high_nibble (c);
		out_buf[hex_map[y]+1] = get_low_nibble (c);
		out_buf[asc_map[y]] = make_printable (c);
	}

	return (bytes_per_line);
}

/* Convert an address to an 8 nibble ascii character string */

format_addr (out_buf, addr)
char	*out_buf;
x_addr	 addr;
{
	int	 y;

	for (y = 0; y < nibbles_per_addr; y++) {
		out_buf[adr_map[y]] = get_low_nibble (addr);
		addr >>= 4;
	}
}

/* Get a byte from the file, return EOF if beyond end of file */

/* Note: calling get_byte_addr with an address beyond end */
/* of file will result in the file being extended to include */
/* the address. This is not desirable for someone who strays */
/* beyond the end of file but has no wish to extend the file. */

int
get_byte (addr)
x_addr	 addr;
{
	x_char	*byte_addr;

	if (addr >= mcb->len)
		return (EOF);

	byte_addr = get_byte_addr (addr);
	return ((int) *byte_addr);
}

/* Update display from this addr on down */

update_page (first_addr, last_addr)
x_addr	 first_addr;
x_addr	 last_addr;
{
	int	 x;
	int	 bc;
	char	 out_buf [OUT_BUF_SZ];
	int	 first_row;
	int	 last_row;

	/* Convert this addr into an offset into the screen */

	first_addr = get_row_addr (first_addr);
	last_addr = get_row_addr (last_addr);

	first_row = (first_addr - base_addr) / bytes_per_line;
	last_row = (last_addr - base_addr) / bytes_per_line;

	/* Format and display each row in the region to be updated */

	for (x = first_row; x < last_row; x++) {
		format_line (out_buf, first_addr);
		display_string (text_row+x, text_col, out_buf, text_attr);
		first_addr += bytes_per_line;
	}
}

/* Display the byte specified by this_addr in its correct */
/* location on the screen and with the specified attribute */

display_byte (this_addr, attr)
x_addr	 this_addr;
int	 attr;
{
	int	 h;
	int	 l;
	int	 c;
	int	 row_offset;
	int	 col_offset;
	x_addr	 offset;

	/* Convert this addr into an offset into the screen */

	offset = this_addr - base_addr;
	row_offset = text_row + (offset / bytes_per_line);
	col_offset = text_col + (offset % bytes_per_line);

	/* Get byte associated with addr from file */

	c = get_byte (this_addr);

	/* Bytes at addres beyond end of file display as spaces */

	if (c == EOF) {
		h = ' ';
		l = ' ';
		c = ' ';
	}

	/* Bytes at other addres are converted to hex format */

	else {
		h = get_high_nibble (c);
		l = get_low_nibble (c);
		c = make_printable (c);
	}

	/* Update screen with new hex and character representations */

	display_char (row_offset, hex_map[col_offset], h, attr);
	display_char (row_offset, hex_map[col_offset]+1, l, attr);
	display_char (row_offset, asc_map[col_offset], c, attr);
}

/* Display a message on the message row */

int	 message_displayed = FALSE;

display_message (buf)
char	*buf;
{
	int	 buf_len;
	int	 center_col;

	/* Calculate column to center the message */

	buf_len = strlen (buf);
	center_col = (maxcol - buf_len) / 2;

	/* Clear the message line if there is anything on it */

	if (message_displayed)
		truncate_row (message_row, message_col, message_attr);

	/* Display the new message and position cursor to home */

	display_string (message_row, center_col, buf, message_attr);
	position_cursor (minrow, mincol);
	message_displayed = TRUE;
}

/* Clear the message row if there is anything on it */

update_message ()
{
	if (message_displayed) {
		truncate_row (message_row, message_col, message_attr);
		message_displayed = FALSE;
	}
}

/* Initialize the screen with the correct attributes */

/* Note: this function supports an inverse video text area */
/* by truncating the message row in the correct attribute. */

init_screen ()
{
	clear_screen (text_attr);
	truncate_row (message_row, message_col, message_attr);
}

/* Print each line of text in usage array and exit */

print_usage ()
{
	int	 i;
	int	 row;

	clear_screen (DEFAULT_ATTR);
	for (i = 0, row = minrow; i < USAGE_SZ; i++) {
		display_string (row, mincol, usage_text[i], DEFAULT_ATTR);
		row++;
	}
	position_cursor (row, mincol);
	restore_hardware ();
	exit (1);
}


